/*******************************************************************************
 * Copyright (c) 2016 Dec. Verimag. All rights reserved.
 * @author Hang YU
 * A polyhedron contains many constraints in the form:
 * a_1 x_1 + a_2 x_2 + ... + a_n x_n <op> b, where <op> is <, <= or ==, and 
 * b is a constant.
 * The matrix _coefficients stores the coefficients of the variables, and the
 * vector _constants stores the constant in each row 
 * _central_point is a point inside of the polyhedron, and the sum of distance 
 * from this point to each facet of the polyhedron is minimized.
 * _active_table is used to mark redundancy now, i.e. its ith value is true if
 * the ith constraint is irredundant. 
*******************************************************************************/

#ifndef _EIGEN
#define _EIGEN
#include <eigen3/Eigen/Dense>
#endif
#include <vector>
#ifndef _POINT
#define _POINT
#include "point.h"
#endif

typedef Eigen::MatrixXd Matrix ;
typedef Eigen::VectorXd Vector ;

class Polyhedron {
public:
  Polyhedron (int consNum, int variNum) ; 
  void Minimize (int method = 1) ;
  int get_variable_num () const ;
  void set_variable_num (int num) ;
  int get_constraints_num () const ;
  void set_constraints_num (int num) ; 
  void Activate (int consIdx) ;
  void Deactivate (int consIdx) ;
  bool IsActive (int consIdx) const ;
  void set_central_point(const Point& point) ;
  Point get_central_point () ; 
  Vector GetConstraint (int consIdx) const ;
  Matrix get_coefficients () const ; 
  void SetConstraint (int consIdx, const Vector& cons) ;
  double GetConstant (int consIdx) const ;
  void SetConstant (int consIdx, double val) ;
  double GetCoef (int consIdx, int variIdx) const ;
  void SetCoef (int consIdx, int variIdx, double val) ;
  void PrintActiveIdx () ;
  void Print () ;
/*
  // TODO
  int AddConstraint (const Vector& constraint) ; 

  // TODO what is the parameter input_point? Is it not the central point?
  // bool sat_constraint(vector* input_point, vector* input_coefs)
  bool Satisfy (const Vector& constraint) ;

  // TODO why we need this? 
  // If the polyhedron does not satisfy any of its constraints, it means the polyhedron is empty
  // TODO what is the parameter input_point? Is it not the central point?
  // bool sat_ith_constraint(vector* input_point, int i)
  bool Satisfy (const Point& point, int i) ;

  bool IsSatisfiedBy (const Point& point) ;
  bool IsConstraintActive (int i) ;
  double Evaluate (const Point& point, int consIdx) ;
  
  // TODO do we need this?
  Vector Evaluate (const Point& point) ;

  bool IsIncludedIn (const Vector& constraint) ;
  bool IsEmpty () const ; 
  void Print (int consIdx) ;
  void PrintSage () ;
  void PrintSage (int consIdx) ;
*/
private:
  // int _id ;
  // the coefficients of the variables 
  Matrix _coefficients ;
  Vector _constants ;
  int _constraints_num ;
  int _variable_num ;
  Point _central_point ;
  std::vector<bool> _active_table ;
  bool _is_empty ;
} ;
